import argparse
import logging
import os
import sys

from azure.storage.blob import BlockBlobService, PublicAccess
import tabulate

from .auxilliaries import tqdmupto

logger = logging.getLogger("azblob")
logger.setLevel(logging.DEBUG)
console_logger_handler = logging.StreamHandler()
console_logger_handler.setLevel(logging.INFO)
console_logger_formatter = logging.Formatter(
    "%(asctime)s|%(name)s|%(levelname)s::%(message)s", "%m-%d@%H:%M"
)
console_logger_handler.setFormatter(console_logger_formatter)
logger.addHandler(console_logger_handler)


def parse_credentials(accountname, accountkey):
    accountname = accountname or os.environ["AZBLOB_ACCOUNTNAME"]
    accountkey = accountkey or os.environ.get("AZBLOB_ACCOUNTKEY")
    return accountname, accountkey


def credentials(f):
    def f_with_credentials(*args, **kwargs):
        kwargs["accountname"], kwargs["accountkey"] = parse_credentials(
            kwargs["accountname"], kwargs["accountkey"]
        )
        logger.info("Azure storage account name: '{}'".format(kwargs["accountname"]))
        if kwargs["accountkey"] is None:
            logger.info("using anonymous access.")
        return f(*args, **kwargs)

    return f_with_credentials


@credentials
def downloadapi(
    container, blob, accountname=None, accountkey=None, replace=True, blob_target=None
):
    block_blob_service = BlockBlobService(
        account_name=accountname, account_key=accountkey
    )
    blob_target = blob_target or os.path.join(os.getcwd(), blob)
    if not replace and os.path.isfile(blob_target):
        logger.info(
            "will skip download, {} already exists and replace=False".format(
                blob_target
            )
        )
        return
    logger.info("downloading '{}/{}' to '{}'".format(container, blob, blob_target))
    with tqdmupto(total=100, ncols=80) as pbar:

        def update(current, total):
            progress = int(100.0 * (current / total) + 0.5)
            pbar.update_to(progress)

        block_blob_service.get_blob_to_path(
            container, blob, blob_target, progress_callback=update
        )


def download(container, blob, accountname=None, accountkey=None, replace=True):
    downloadapi(
        container,
        blob,
        accountname=None,
        accountkey=None,
        replace=replace,
        blob_target=None,
    )


@credentials
def listblobsapi(container, accountname=None, accountkey=None, nmax=None):
    block_blob_service = BlockBlobService(
        account_name=accountname, account_key=accountkey
    )
    logger.info("listing blobs in '{}/{}'".format(accountname, container))
    if nmax is None:
        nmax = sys.maxsize
    blobs = block_blob_service.list_blobs(container)
    blob_list = [
        {"name": blob.name, "date": blob.properties.creation_time}
        for i, blob in enumerate(blobs)
        if i < nmax
    ]
    return blob_list


def listblobs(container, accountname=None, accountkey=None, nmax=None):
    blobs = listblobsapi(
        container, accountname=accountname, accountkey=accountkey, nmax=nmax
    )
    table_header = ("Name", "Date")
    table_rows = [(blob["name"], blob["date"]) for blob in blobs]
    print(tabulate.tabulate(table_rows, headers=table_header))


def cli():
    # azblob
    parser = argparse.ArgumentParser(
        description="minimal Azure blob storage operations"
    )
    parser.add_argument("-n", "--accountname", default=None)
    parser.add_argument("-k", "--accountkey", default=None)

    subparsers = parser.add_subparsers(dest="operation", help="blob operations")

    # azblob download
    parser_get = subparsers.add_parser("download", help="download blob")
    parser_get.add_argument("container", help="container name")
    parser_get.add_argument("blob", help="blob name (file name)")
    parser_get.add_argument(
        "--dontreplace",
        action="store_true",
        help="Check if target download path exists and if so then dont download.",
    )

    # azblob list
    parser_get = subparsers.add_parser("list", help="list blobs in container")
    parser_get.add_argument("container", help="container name")
    parser_get.add_argument(
        "--nmax", type=int, help="maximum number of blobs to list", default=10
    )

    args = parser.parse_args()
    logger.info("cli args: op={}".format(args.operation))

    if args.operation == "download":
        download(
            args.container,
            args.blob,
            accountname=args.accountname,
            accountkey=args.accountkey,
            replace=not args.dontreplace,
        )
    elif args.operation == "list":
        listblobs(
            args.container,
            accountname=args.accountname,
            accountkey=args.accountkey,
            nmax=args.nmax,
        )


if __name__ == "__main__":
    cli()



from tqdm import tqdm


class tqdmupto(tqdm):
    """Provides `update_to(n)` which uses `tqdm.update(delta_n)`.
    
    
    From https://github.com/tqdm/tqdm 
    """

    def update_to(self, b=1, bsize=1, tsize=None):
        """
        b  : int, optional
            Number of blocks transferred so far [default: 1].
        bsize  : int, optional
            Size of each block (in tqdm units) [default: 1].
        tsize  : int, optional
            Total size (in tqdm units). If [default: None] remains unchanged.
        """
        if tsize is not None:
            self.total = tsize
        self.update(b * bsize - self.n)  # will also set self.n = b * bsize



from setuptools import setup


setup(
    name="azblob",
    version="0.0.5",
    author="Philipp Lang",
    packages=["azblob"],
    url=("https://github.com/plang85/azblob"),
    license="MIT License",
    description="Download Azure blobs.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development",
    ],
    entry_points={"console_scripts": ["azblob = azblob.ops:cli"]},
    install_requires=[
        "azure-storage-blob>=1.3.0",
        "azure-storage-file>=1.3.0",
        "tqdm>=4.0.0",
        "tabulate>=0.8.2",
    ],
    extras_require={"dev": ["black", "twine"]},
)
